#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <limits.h>
#include <string.h>
#include <unistd.h>
#include <inttypes.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "lfsr.h"

#define BUFF_SIZE (1024*1024*2)

void usage(){
	fprintf(stderr, "Usage: ./program <seed> <spin> <path> <size>\n");
}

int main(int argc, char *argv[]) {
	int r;
	int rv;
	struct fio_lfsr *fl;
	unsigned int spin = 0;
	uint64_t seed = 0;
	uint64_t written = 0;
	uint64_t write_size = BUFF_SIZE;
	uint64_t i;
	uint64_t * buffer;
	uint64_t size = 0;
	char *path;

	/* Read arguments */
	if(argc == 5) {
		size = atoll(argv[4]);
		path = argv[3];
		spin = atoi(argv[2]);
		seed = atol(argv[1]);
	} else {
		usage();
		return 1;
	}

	/* Initialize */
	fl = malloc(sizeof(struct fio_lfsr));
	if (!fl) {
		perror("malloc");
		return 1;
	}

	buffer = malloc(BUFF_SIZE);
	if (!buffer) {
		perror("malloc");
		return 1;
	}

	r = lfsr_init(fl, LLONG_MAX, seed, spin);
	if (r) {
		printf("Initialization failed.\n");
		return r;
	}

	/*
	 * Iterate over a tight loop until we have produced all the requested
	 * numbers.
	 */
	int j = 0;
	int fd = open(path, O_WRONLY|O_CREAT|O_TRUNC, S_IRUSR|S_IWUSR);
	if ( fd < 0 ) {
		perror("open");
		return fd;
	}

	while (written < size) {
		while (written + write_size > size) {
			write_size=write_size/2;
		}
			
		for (j=0; j<write_size/sizeof(int64_t); j++) {
			lfsr_next(fl, &i, fl->max_val);
			buffer[j] = i;
		}
		rv = write(fd, buffer, write_size);
		if (rv < 0 ) {
			perror("write");
			goto out;
		}
		if ( rv != write_size ) {
			fprintf(stderr, "warning, not full buffer written!\n");
		}
		written += rv;
			
	}
//	fprintf(stderr, "finished.\n");
out:
	close(fd);
	free(fl);
	return r;
}
